local UIAnim = require "widgets/uianim"
local Text = require "widgets/text"
local Widget = require "widgets/widget"
local Image = require "widgets/image"
local ImageButton = require "widgets/imagebutton"
local AnimButton = require "widgets/animbutton"
local HoverText = require "widgets/hoverer"

local ManaIndicator = Class(Widget, function(self, owner)
	Widget._ctor(self, "ManaIndicator")
	self.owner = owner
	self.scale = 1
	
	self.manaindicator = self:AddChild(Widget("manaindicator"))
	self.manaindicator:SetPosition(0, 0,0)
	self.manaindicator:SetScale(0.3,0.3,0.3)	

	
	self.manaindicator.text = self:AddChild(Text(BODYTEXTFONT, 35))
	self.manaindicator.text:SetHAlign(ANCHOR_MIDDLE)
	self.manaindicator.text:SetPosition(5, 0, 0)
	self.manaindicator.text:MoveToFront()
	self.manaindicator.text.current = owner.currentmana:value()
	self.manaindicator.text.max = 200

	
	self.ManaPercent = 1 - self.manaindicator.text.current/self.manaindicator.text.max or 0
	self.manaindicator.anim = self.manaindicator:AddChild(UIAnim())
	self.manaindicator.anim:GetAnimState():SetBank("manaindicator")
	self.manaindicator.anim:GetAnimState():SetBuild("manaindicator")
	self.manaindicator.anim:GetAnimState():PlayAnimation("anim")
	self.manaindicator.anim:GetAnimState():SetPercent("anim", self.ManaPercent)
	
	self:StartUpdating()
end)


function ManaIndicator:OnUpdate(dt)
	local currentmana = self.owner.currentmana:value()
	local percent = currentmana/self.manaindicator.text.max
	self.manaindicator.anim:GetAnimState():SetPercent("anim",1 - percent)
	local manistr = string.format("%3d/%3d\n能量",currentmana,self.manaindicator.text.max)
	self.manaindicator.text:SetString(manistr)
end

function ManaIndicator:OnControl (control, down)
	if control == CONTROL_ACCEPT then
		if down then
			self:StartDrag()
		else
			self:EndDrag()
		end
	elseif control == CONTROL_SCROLLBACK then
		self:Scale_DoDelta(-0.05)
	elseif control == CONTROL_SCROLLFWD then
		self:Scale_DoDelta(0.05)
	end
end
function ManaIndicator:SetDragPosition(x, y, z)

	local pos
	if type(x) == "number" then
		pos = Vector3(x, y, z)
	else
		pos = x
	end
	self:SetPosition(pos + self.dragPosDiff)
end
function ManaIndicator:StartDrag()
	-- based on Widget:FollowMouse()
	if not self.followhandler then
		local mousepos = TheInput:GetScreenPosition()
		self.dragPosDiff = self:GetPosition() - mousepos
		self.followhandler = TheInput:AddMoveHandler(function(x,y) self:SetDragPosition(x,y) end)
		self:SetDragPosition(mousepos)
	end
end

function ManaIndicator:EndDrag()
	-- based on Widget:StopFollowMouse()
	if self.followhandler then
		self.followhandler:Remove()
	end
	self.followhandler = nil
	self.dragPosDiff = nil
end
function ManaIndicator:Scale_DoDelta(delta)
	self.scale = math.max(self.scale+delta,0.1)
	print("self.scale",self.scale)
	self:SetScale(self.scale,self.scale,self.scale)
end


return ManaIndicator